"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var Promise = require("bluebird");
var api_1 = require("./api");
var memory_fs_1 = require("./memory-fs");
var utils_1 = require("./utils");
function nodesToMap(tree, accumulator) {
    if (accumulator === void 0) { accumulator = {}; }
    tree.forEach(function (node) {
        if (api_1.isDir(node))
            nodesToMap(node.children, accumulator);
        accumulator[node.fullPath] = node;
    });
    return accumulator;
}
;
function getTreesDiff(cached, real) {
    var diff = {
        toAdd: [],
        toDelete: [],
        toChange: []
    };
    Object.keys(cached).forEach(function (cachedPath) {
        if (!real[cachedPath]) {
            diff.toDelete.push(cached[cachedPath]);
            diff.toChange = diff.toChange.filter(function (path) { return path !== cachedPath; });
        }
        else {
            var node = cached[cachedPath];
            if (api_1.isFile(node) && node.content)
                diff.toChange.push(cachedPath);
        }
    });
    Object.keys(real).forEach(function (realPath) {
        if (!cached[realPath])
            diff.toAdd.push(real[realPath]);
    });
    return diff;
}
var CacheFileSystem = (function () {
    function CacheFileSystem(fs, shouldRescanOnError) {
        if (shouldRescanOnError === void 0) { shouldRescanOnError = true; }
        var _this = this;
        this.fs = fs;
        this.shouldRescanOnError = shouldRescanOnError;
        this.events = utils_1.makeEventsEmitter();
        this.isTreeCached = false;
        this.pathsInCache = {};
        this.onFsError = function (_a) {
            var stack = _a.stack;
            _this.shouldRescanOnError ?
                _this.rescanOnError() :
                _this.emit('unexpectedError', { stack: stack });
        };
        this.baseUrl = fs.baseUrl;
        this.cache = new memory_fs_1.MemoryFileSystem();
        this.fs.events.on('unexpectedError', this.onFsError);
        this.fs.events.on('fileCreated', function (event) {
            var fullPath = event.fullPath, newContent = event.newContent;
            try {
                _this.cache.saveFileSync(fullPath, newContent);
                _this.pathsInCache[fullPath] = true;
                _this.events.emit('fileCreated', event);
            }
            catch (e) {
                _this.onFsError(e);
            }
        });
        this.fs.events.on('fileChanged', function (event) {
            var fullPath = event.fullPath, newContent = event.newContent;
            try {
                _this.cache.saveFileSync(fullPath, newContent);
                _this.pathsInCache[fullPath] = true;
                _this.events.emit('fileChanged', event);
            }
            catch (e) {
                _this.onFsError(e);
            }
        });
        this.fs.events.on('fileDeleted', function (event) {
            try {
                _this.cache.deleteFileSync(event.fullPath);
                _this.pathsInCache[event.fullPath] = true;
                _this.events.emit('fileDeleted', event);
            }
            catch (e) {
                _this.onFsError(e);
            }
        });
        this.fs.events.on('directoryCreated', function (event) {
            try {
                _this.cache.ensureDirectorySync(event.fullPath);
                _this.events.emit('directoryCreated', event);
            }
            catch (e) {
                _this.onFsError(e);
            }
        });
        this.fs.events.on('directoryDeleted', function (event) {
            try {
                _this.cache.deleteDirectorySync(event.fullPath, true);
                _this.events.emit('directoryDeleted', event);
            }
            catch (e) {
                _this.onFsError(e);
            }
        });
    }
    CacheFileSystem.prototype.saveFile = function (fullPath, newContent) {
        var _this = this;
        return this.fs.saveFile(fullPath, newContent)
            .then(function () { return _this.cache.saveFile(fullPath, newContent); })
            .then(function () {
            _this.pathsInCache[fullPath] = true;
        });
    };
    CacheFileSystem.prototype.deleteFile = function (fullPath) {
        var _this = this;
        return this.fs.deleteFile(fullPath)
            .then(function () { return _this.cache.deleteFile(fullPath); });
    };
    CacheFileSystem.prototype.deleteDirectory = function (fullPath, recursive) {
        var _this = this;
        if (recursive === void 0) { recursive = false; }
        return this.fs.deleteDirectory(fullPath, recursive)
            .then(function () { return _this.cache.deleteDirectory(fullPath, recursive); });
    };
    CacheFileSystem.prototype.ensureDirectory = function (fullPath) {
        var _this = this;
        return this.fs.ensureDirectory(fullPath)
            .then(function () { return _this.cache.ensureDirectory(fullPath); });
    };
    CacheFileSystem.prototype.loadTextFile = function (fullPath) {
        var _this = this;
        if (this.pathsInCache[fullPath])
            return this.cache.loadTextFile(fullPath);
        return this.fs.loadTextFile(fullPath)
            .then(function (file) {
            return _this.cache.saveFile(fullPath, file)
                .then(function () { return _this.pathsInCache[fullPath] = true; })
                .then(function () { return _this.loadTextFile(fullPath); });
        });
    };
    CacheFileSystem.prototype.loadDirectoryTree = function (fullPath) {
        var _this = this;
        if (this.isTreeCached)
            return this.cache.loadDirectoryTree(fullPath);
        return this.cacheTree().then(function () {
            _this.isTreeCached = true;
            return _this.loadDirectoryTree(fullPath);
        });
    };
    CacheFileSystem.prototype.loadDirectoryChildren = function (fullPath) {
        var _this = this;
        if (this.isTreeCached)
            return this.cache.loadDirectoryChildren(fullPath);
        return this.cacheTree().then(function () {
            _this.isTreeCached = true;
            return _this.loadDirectoryChildren(fullPath);
        });
    };
    CacheFileSystem.prototype.dispose = function () {
        if (api_1.isDisposable(this.fs))
            this.fs.dispose();
    };
    CacheFileSystem.prototype.rescanOnError = function () {
        var _this = this;
        var cachedTree = this.cache.loadDirectoryTreeSync();
        this.isTreeCached = false;
        this.loadDirectoryTree().then(function (realTree) {
            var _a = getTreesDiff(nodesToMap(cachedTree.children), nodesToMap(realTree.children)), toDelete = _a.toDelete, toAdd = _a.toAdd, toChange = _a.toChange;
            toDelete.forEach(function (node) {
                _this.emit((api_1.isDir(node) ? 'directory' : 'file') + "Deleted", { fullPath: node.fullPath });
            });
            toAdd.forEach(function (node) {
                if (api_1.isDir(node)) {
                    _this.emit('directoryCreated', { fullPath: node.fullPath });
                }
                else {
                    _this.loadTextFile(node.fullPath).then(function (newContent) {
                        _this.emit('fileCreated', {
                            fullPath: node.fullPath,
                            newContent: newContent
                        });
                    });
                }
            });
            toChange.forEach(function (fullPath) { return _this.loadTextFile(fullPath).then(function (newContent) {
                _this.emit('fileChanged', { fullPath: fullPath, newContent: newContent });
            }); });
        });
    };
    CacheFileSystem.prototype.emit = function (type, data) {
        this.events.emit(type, __assign({}, data, { type: type }));
    };
    CacheFileSystem.prototype.cacheTree = function () {
        var _this = this;
        this.cache = new memory_fs_1.MemoryFileSystem();
        this.pathsInCache = {};
        return this.fs.loadDirectoryTree()
            .then(function (tree) { return _this.fill(tree); });
    };
    CacheFileSystem.prototype.fill = function (tree) {
        var _this = this;
        if (api_1.isDir(tree)) {
            return this.cache.ensureDirectory(tree.fullPath)
                .then(function () { return Promise.all(tree.children.map(function (child) { return _this.fill(child); })); })
                .then(function () { return _this.cache; });
        }
        return this.cache.saveFile(tree.fullPath, '')
            .then(function () { return Promise.resolve(_this.cache); });
    };
    return CacheFileSystem;
}());
exports.CacheFileSystem = CacheFileSystem;
//# sourceMappingURL=cache-fs.js.map