"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Promise = require("bluebird");
var autobahn_1 = require("autobahn");
var api_1 = require("./api");
var utils_1 = require("./utils");
exports.noConnectionError = "WampClientFileSystem hasn't opened connection yet (forgot to init()?).";
var WampClientFileSystem = (function () {
    function WampClientFileSystem(url, realm, initTimeout) {
        if (initTimeout === void 0) { initTimeout = 5000; }
        this.realm = realm;
        this.initTimeout = initTimeout;
        this.events = utils_1.makeEventsEmitter();
        this.baseUrl = url;
        this.realm = realm;
        this.connection = new autobahn_1.Connection({ url: url, realm: realm });
    }
    WampClientFileSystem.prototype.init = function () {
        var _this = this;
        var _a = this, baseUrl = _a.baseUrl, initTimeout = _a.initTimeout, connection = _a.connection;
        return new Promise(function (resolve) {
            connection.open();
            connection.onopen = function (session) {
                _this.session = session;
                _this.realmPrefix = _this.realm.replace(/(.*\..*)(\..*)$/, '$1.'); // 'xxx.yyy.zzz' => 'xxx.yyy.'
                api_1.fileSystemEventNames.forEach(function (fsEvent) {
                    _this.session.subscribe(_this.realmPrefix + fsEvent, function (res) { return _this.events.emit(fsEvent, res && res[0]); });
                });
                return resolve(_this);
            };
        }).timeout(initTimeout, "Cant't open connection to the WAMP server at " + baseUrl + " for " + initTimeout + "ms.");
    };
    WampClientFileSystem.prototype.saveFile = function (fullPath, newContent) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            if (!_this.session || !_this.session.isOpen) {
                return reject(exports.noConnectionError);
            }
            _this.session.call(_this.realmPrefix + "saveFile", [fullPath, newContent])
                .then(function () { return resolve(); })
                .catch(function (error) { return reject(new Error(error)); });
        });
    };
    WampClientFileSystem.prototype.deleteFile = function (fullPath) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            if (!_this.session || !_this.session.isOpen) {
                return reject(exports.noConnectionError);
            }
            _this.session.call(_this.realmPrefix + "deleteFile", [fullPath])
                .then(function () { return resolve(); })
                .catch(function (error) { return reject(new Error(error)); });
        });
    };
    WampClientFileSystem.prototype.deleteDirectory = function (fullPath, recursive) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            if (!_this.session || !_this.session.isOpen) {
                return reject(exports.noConnectionError);
            }
            _this.session.call(_this.realmPrefix + "deleteDirectory", [fullPath, recursive])
                .then(function () { return resolve(); })
                .catch(function (error) { return reject(new Error(error)); });
        });
    };
    WampClientFileSystem.prototype.ensureDirectory = function (fullPath) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            if (!_this.session || !_this.session.isOpen) {
                return reject(exports.noConnectionError);
            }
            _this.session.call(_this.realmPrefix + "ensureDirectory", [fullPath])
                .then(function () { return resolve(); })
                .catch(function (error) { return reject(new Error(error)); });
        });
    };
    WampClientFileSystem.prototype.loadTextFile = function (fullPath) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            if (!_this.session || !_this.session.isOpen) {
                return reject(exports.noConnectionError);
            }
            return _this.session.call(_this.realmPrefix + "loadTextFile", [fullPath])
                .then(function (content) { return resolve(content); })
                .catch(function (error) { return reject(new Error(error)); });
        });
    };
    WampClientFileSystem.prototype.loadDirectoryTree = function (fullPath) {
        var _this = this;
        if (fullPath === void 0) { fullPath = ''; }
        return new Promise(function (resolve, reject) {
            if (!_this.session || !_this.session.isOpen) {
                return reject(exports.noConnectionError);
            }
            return _this.session.call(_this.realmPrefix + "loadDirectoryTree", [fullPath])
                .then(function (tree) { return resolve(tree); })
                .catch(function (error) { return reject(new Error(error)); });
        });
    };
    WampClientFileSystem.prototype.loadDirectoryChildren = function (fullPath) {
        var _this = this;
        return Promise.try(function () {
            if (!_this.session || !_this.session.isOpen) {
                throw new Error(exports.noConnectionError);
            }
            return _this.session.call(_this.realmPrefix + "loadDirectoryChildren", [fullPath])
                .catch(function (error) { throw new Error(error); });
        });
    };
    WampClientFileSystem.prototype.dispose = function () {
        this.connection && this.connection.close();
    };
    return WampClientFileSystem;
}());
exports.WampClientFileSystem = WampClientFileSystem;
//# sourceMappingURL=wamp-client-fs.js.map